@@ -2,20 +2,10 @@
 #
 # This file is part of Glances.
 #
-# Copyright (C) 2021 Nicolargo <nicolas@nicolargo.com>
+# SPDX-FileCopyrightText: 2022 Nicolas Hennion <nicolas@nicolargo.com>
 #
-# Glances is free software; you can redistribute it and/or modify
-# it under the terms of the GNU Lesser General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# SPDX-License-Identifier: LGPL-3.0-only
 #
-# Glances is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public License
-# along with this program. If not, see <http://www.gnu.org/licenses/>.
 
 """Manage autodiscover Glances server (thk to the ZeroConf protocol)."""
 
@@ -26,12 +16,8 @@ from glances.globals import BSD
 from glances.logger import logger
 
 try:
-    from zeroconf import (
-        __version__ as __zeroconf_version,
-        ServiceBrowser,
-        ServiceInfo,
-        Zeroconf
-    )
+    from zeroconf import __version__ as __zeroconf_version, ServiceBrowser, ServiceInfo, Zeroconf
+
     zeroconf_tag = True
 except ImportError:
     zeroconf_tag = False
@@ -46,7 +32,7 @@ if zeroconf_tag:
         sys.exit(1)
 
 # Global var
-# Recent versions of the zeroconf python package doesnt like a zeroconf type that ends with '._tcp.'.
+# Recent versions of the zeroconf python package doesn't like a zeroconf type that ends with '._tcp.'.
 # Correct issue: zeroconf problem with zeroconf_type = "_%s._tcp." % 'glances' #888
 zeroconf_type = "_%s._tcp.local." % 'glances'
 
@@ -78,10 +64,10 @@ class AutoDiscovered(object):
             'username': 'glances',  # Default username
             'password': '',  # Default password
             'status': 'UNKNOWN',  # Server status: 'UNKNOWN', 'OFFLINE', 'ONLINE', 'PROTECTED'
-            'type': 'DYNAMIC'}  # Server type: 'STATIC' or 'DYNAMIC'
+            'type': 'DYNAMIC',
+        }  # Server type: 'STATIC' or 'DYNAMIC'
         self._server_list.append(new_server)
-        logger.debug("Updated servers list (%s servers): %s" %
-                     (len(self._server_list), self._server_list))
+        logger.debug("Updated servers list (%s servers): %s" % (len(self._server_list), self._server_list))
 
     def remove_server(self, name):
         """Remove a server from the dict."""
@@ -90,11 +76,9 @@ class AutoDiscovered(object):
                 try:
                     self._server_list.remove(i)
                     logger.debug("Remove server %s from the list" % name)
-                    logger.debug("Updated servers list (%s servers): %s" % (
-                        len(self._server_list), self._server_list))
+                    logger.debug("Updated servers list (%s servers): %s" % (len(self._server_list), self._server_list))
                 except ValueError:
-                    logger.error(
-                        "Cannot remove server %s from the list" % name)
+                    logger.error("Cannot remove server %s from the list" % name)
 
 
 class GlancesAutoDiscoverListener(object):
@@ -116,32 +100,30 @@ class GlancesAutoDiscoverListener(object):
     def add_service(self, zeroconf, srv_type, srv_name):
         """Method called when a new Zeroconf client is detected.
 
-        Return True if the zeroconf client is a Glances server
         Note: the return code will never be used
+
+        :return: True if the zeroconf client is a Glances server
         """
         if srv_type != zeroconf_type:
             return False
-        logger.debug("Check new Zeroconf server: %s / %s" %
-                     (srv_type, srv_name))
+        logger.debug("Check new Zeroconf server: %s / %s" % (srv_type, srv_name))
         info = zeroconf.get_service_info(srv_type, srv_name)
-        if info:
-            new_server_ip = socket.inet_ntoa(info.address)
+        if info and (info.addresses or info.parsed_addresses):
+            address = info.addresses[0] if info.addresses else info.parsed_addresses[0]
+            new_server_ip = socket.inet_ntoa(address)
             new_server_port = info.port
 
             # Add server to the global dict
             self.servers.add_server(srv_name, new_server_ip, new_server_port)
-            logger.info("New Glances server detected (%s from %s:%s)" %
-                        (srv_name, new_server_ip, new_server_port))
+            logger.info("New Glances server detected (%s from %s:%s)" % (srv_name, new_server_ip, new_server_port))
         else:
-            logger.warning(
-                "New Glances server detected, but Zeroconf info failed to be grabbed")
+            logger.warning("New Glances server detected, but failed to be get Zeroconf ServiceInfo ")
         return True
 
     def remove_service(self, zeroconf, srv_type, srv_name):
         """Remove the server from the list."""
         self.servers.remove_server(srv_name)
-        logger.info(
-            "Glances server %s removed from the autodetect list" % srv_name)
+        logger.info("Glances server %s removed from the autodetect list" % srv_name)
 
 
 class GlancesAutoDiscoverServer(object):
@@ -158,8 +140,7 @@ class GlancesAutoDiscoverServer(object):
                 self.zeroconf_enable_tag = False
             else:
                 self.listener = GlancesAutoDiscoverListener()
-                self.browser = ServiceBrowser(
-                    self.zeroconf, zeroconf_type, self.listener)
+                self.browser = ServiceBrowser(self.zeroconf, zeroconf_type, self.listener)
                 self.zeroconf_enable_tag = True
         else:
             logger.error("Cannot start autodiscover mode (Zeroconf lib is not installed)")
@@ -205,6 +186,9 @@ class GlancesAutoDiscoverClient(object):
                     # Issue #528 (no network interface available)
                     pass
 
+            # Ensure zeroconf_bind_address is an IP address not an host
+            zeroconf_bind_address = socket.gethostbyname(zeroconf_bind_address)
+
             # Check IP v4/v6
             address_family = socket.getaddrinfo(zeroconf_bind_address, args.port)[0][0]
 
@@ -212,29 +196,27 @@ class GlancesAutoDiscoverClient(object):
             try:
                 self.info = ServiceInfo(
                     zeroconf_type,
-                    '{}:{}.{}'.format(hostname,
-                                      args.port,
-                                      zeroconf_type),
+                    '{}:{}.{}'.format(hostname, args.port, zeroconf_type),
                     address=socket.inet_pton(address_family, zeroconf_bind_address),
                     port=args.port,
                     weight=0,
                     priority=0,
                     properties={},
-                    server=hostname)
+                    server=hostname,
+                )
             except TypeError:
                 # Manage issue 1663 with breaking change on ServiceInfo method
                 # address (only one address) is replaced by addresses (list of addresses)
                 self.info = ServiceInfo(
                     zeroconf_type,
-                    name = '{}:{}.{}'.format(hostname,
-                                             args.port,
-                                             zeroconf_type),
+                    name='{}:{}.{}'.format(hostname, args.port, zeroconf_type),
                     addresses=[socket.inet_pton(address_family, zeroconf_bind_address)],
                     port=args.port,
                     weight=0,
                     priority=0,
                     properties={},
-                    server=hostname)
+                    server=hostname,
+                )
             try:
                 self.zeroconf.register_service(self.info)
             except Exception as e:
@@ -248,6 +230,7 @@ class GlancesAutoDiscoverClient(object):
     def find_active_ip_address():
         """Try to find the active IP addresses."""
         import netifaces
+
         # Interface of the default gateway
         gateway_itf = netifaces.gateways()['default'][netifaces.AF_INET][1]
         # IP address for the interface
