@@ -2,20 +2,10 @@
 #
 # This file is part of Glances.
 #
-# Copyright (C) 2021 Nicolargo <nicolas@nicolargo.com>
+# SPDX-FileCopyrightText: 2022 Nicolas Hennion <nicolas@nicolargo.com>
 #
-# Glances is free software; you can redistribute it and/or modify
-# it under the terms of the GNU Lesser General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# SPDX-License-Identifier: LGPL-3.0-only
 #
-# Glances is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public License
-# along with this program. If not, see <http://www.gnu.org/licenses/>.
 
 """InfluxDB (from to InfluxDB 1.8+) interface class."""
 
@@ -35,27 +25,25 @@ class Export(GlancesExport):
         """Init the InfluxDB export IF."""
         super(Export, self).__init__(config=config, args=args)
 
-        # Mandatories configuration keys (additional to host and port)
+        # Mandatory configuration keys (additional to host and port)
         self.org = None
         self.bucket = None
         self.token = None
 
-        # Optionals configuration keys
+        # Optional configuration keys
         self.protocol = 'http'
         self.prefix = None
         self.tags = None
         self.hostname = None
 
         # Load the InfluxDB configuration file
-        self.export_enable = self.load_conf('influxdb2',
-                                            mandatories=['host', 'port',
-                                                         'user', 'password',
-                                                         'org', 'bucket', 'token'],
-                                            options=['protocol',
-                                                     'prefix',
-                                                     'tags'])
+        self.export_enable = self.load_conf(
+            'influxdb2',
+            mandatories=['host', 'port', 'user', 'password', 'org', 'bucket', 'token'],
+            options=['protocol', 'prefix', 'tags'],
+        )
         if not self.export_enable:
-            sys.exit(2)
+            exit('Missing INFLUXDB version 1 config')
 
         # The hostname is always add as a tag
         self.hostname = node().split('.')[0]
@@ -71,30 +59,34 @@ class Export(GlancesExport):
         url = '{}://{}:{}'.format(self.protocol, self.host, self.port)
         try:
             # See docs: https://influxdb-client.readthedocs.io/en/stable/api.html#influxdbclient
-            client = InfluxDBClient(url=url,
-                                    enable_gzip=False,
-                                    verify_ssl=False,
-                                    org=self.org,
-                                    token=self.token)
+            client = InfluxDBClient(url=url, enable_gzip=False, verify_ssl=False, org=self.org, token=self.token)
         except Exception as e:
             logger.critical("Cannot connect to InfluxDB server '%s' (%s)" % (url, e))
             sys.exit(2)
         else:
-            logger.info("Connected to InfluxDB server version {} ({})".format(client.health().version,
-                                                                              client.health().message))
+            logger.info(
+                "Connected to InfluxDB server version {} ({})".format(client.health().version, client.health().message)
+            )
 
         # Create the write client
-        write_client = client.write_api(write_options=WriteOptions(batch_size=500,
-                                                                   flush_interval=10000,
-                                                                   jitter_interval=2000,
-                                                                   retry_interval=5000,
-                                                                   max_retries=5,
-                                                                   max_retry_delay=30000,
-                                                                   exponential_base=2))
+        write_client = client.write_api(
+            write_options=WriteOptions(
+                batch_size=500,
+                flush_interval=10000,
+                jitter_interval=2000,
+                retry_interval=5000,
+                max_retries=5,
+                max_retry_delay=30000,
+                exponential_base=2,
+            )
+        )
         return write_client
 
     def _normalize(self, name, columns, points):
-        """Normalize data for the InfluxDB's data model."""
+        """Normalize data for the InfluxDB's data model.
+
+        :return: a list of measurements.
+        """
         ret = []
 
         # Build initial dict by crossing columns and point
@@ -109,9 +101,11 @@ class Export(GlancesExport):
         for measurement in keys_list:
             # Manage field
             if measurement is not None:
-                fields = {k.replace('{}.'.format(measurement), ''): data_dict[k]
-                          for k in data_dict
-                          if k.startswith('{}.'.format(measurement))}
+                fields = {
+                    k.replace('{}.'.format(measurement), ''): data_dict[k]
+                    for k in data_dict
+                    if k.startswith('{}.'.format(measurement))
+                }
             else:
                 fields = data_dict
             # Transform to InfluxDB datamodel
@@ -140,9 +134,7 @@ class Export(GlancesExport):
             # Add the hostname as a tag
             tags['hostname'] = self.hostname
             # Add the measurement to the list
-            ret.append({'measurement': name,
-                        'tags': tags,
-                        'fields': fields})
+            ret.append({'measurement': name, 'tags': tags, 'fields': fields})
         return ret
 
     def export(self, name, columns, points):
@@ -155,10 +147,7 @@ class Export(GlancesExport):
             logger.debug("Cannot export empty {} stats to InfluxDB".format(name))
         else:
             try:
-                self.client.write(self.bucket,
-                                  self.org,
-                                  self._normalize(name, columns, points),
-                                  time_precision="s")
+                self.client.write(self.bucket, self.org, self._normalize(name, columns, points), time_precision="s")
             except Exception as e:
                 # Log level set to debug instead of error (see: issue #1561)
                 logger.debug("Cannot export {} stats to InfluxDB ({})".format(name, e))
