@@ -2,20 +2,10 @@
 #
 # This file is part of Glances.
 #
-# Copyright (C) 2019 Nicolargo <nicolas@nicolargo.com>
+# SPDX-FileCopyrightText: 2022 Nicolas Hennion <nicolas@nicolargo.com>
 #
-# Glances is free software; you can redistribute it and/or modify
-# it under the terms of the GNU Lesser General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# SPDX-License-Identifier: LGPL-3.0-only
 #
-# Glances is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public License
-# along with this program. If not, see <http://www.gnu.org/licenses/>.
 
 """Monitor plugin."""
 
@@ -29,16 +19,14 @@ class Plugin(GlancesPlugin):
 
     def __init__(self, args=None, config=None):
         """Init the plugin."""
-        super(Plugin, self).__init__(args=args,
-                                     config=config,
-                                     stats_init_value=[])
+        super(Plugin, self).__init__(args=args, config=config, stats_init_value=[])
         self.args = args
         self.config = config
 
         # We want to display the stat in the curse interface
         self.display_curse = True
 
-        # Init the list of AMP (classe define in the glances/amps_list.py script)
+        # Init the list of AMP (classes define in the glances/amps_list.py script)
         self.glances_amps = glancesAmpsList(self.args, self.config)
 
     def get_key(self):
@@ -54,16 +42,19 @@ class Plugin(GlancesPlugin):
 
         if self.input_method == 'local':
             for k, v in iteritems(self.glances_amps.update()):
-                stats.append({'key': self.get_key(),
-                              'name': v.NAME,
-                              'result': v.result(),
-                              'refresh': v.refresh(),
-                              'timer': v.time_until_refresh(),
-                              'count': v.count(),
-                              'countmin': v.count_min(),
-                              'countmax': v.count_max(),
-                              'regex': v.regex() is not None},
-                              )
+                stats.append(
+                    {
+                        'key': self.get_key(),
+                        'name': v.NAME,
+                        'result': v.result(),
+                        'refresh': v.refresh(),
+                        'timer': v.time_until_refresh(),
+                        'count': v.count(),
+                        'countmin': v.count_min(),
+                        'countmax': v.count_max(),
+                        'regex': v.regex() is not None,
+                    },
+                )
         else:
             # Not available in SNMP mode
             pass
