@@ -2,20 +2,10 @@
 #
 # This file is part of Glances.
 #
-# Copyright (C) 2021 Nicolargo <nicolas@nicolargo.com>
+# SPDX-FileCopyrightText: 2022 Nicolas Hennion <nicolas@nicolargo.com>
 #
-# Glances is free software; you can redistribute it and/or modify
-# it under the terms of the GNU Lesser General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# SPDX-License-Identifier: LGPL-3.0-only
 #
-# Glances is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public License
-# along with this program. If not, see <http://www.gnu.org/licenses/>.
 
 """Connections plugin."""
 from __future__ import unicode_literals
@@ -41,25 +31,29 @@ class Plugin(GlancesPlugin):
     stats is a dict
     """
 
-    status_list = [psutil.CONN_LISTEN,
-                   psutil.CONN_ESTABLISHED]
-    initiated_states = [psutil.CONN_SYN_SENT,
-                        psutil.CONN_SYN_RECV]
-    terminated_states = [psutil.CONN_FIN_WAIT1,
-                         psutil.CONN_FIN_WAIT2,
-                         psutil.CONN_TIME_WAIT,
-                         psutil.CONN_CLOSE,
-                         psutil.CONN_CLOSE_WAIT,
-                         psutil.CONN_LAST_ACK]
-    conntrack = {'nf_conntrack_count': '/proc/sys/net/netfilter/nf_conntrack_count',
-                 'nf_conntrack_max': '/proc/sys/net/netfilter/nf_conntrack_max'}
+    status_list = [psutil.CONN_LISTEN, psutil.CONN_ESTABLISHED]
+    initiated_states = [psutil.CONN_SYN_SENT, psutil.CONN_SYN_RECV]
+    terminated_states = [
+        psutil.CONN_FIN_WAIT1,
+        psutil.CONN_FIN_WAIT2,
+        psutil.CONN_TIME_WAIT,
+        psutil.CONN_CLOSE,
+        psutil.CONN_CLOSE_WAIT,
+        psutil.CONN_LAST_ACK,
+    ]
+    conntrack = {
+        'nf_conntrack_count': '/proc/sys/net/netfilter/nf_conntrack_count',
+        'nf_conntrack_max': '/proc/sys/net/netfilter/nf_conntrack_max',
+    }
 
     def __init__(self, args=None, config=None):
         """Init the plugin."""
-        super(Plugin, self).__init__(args=args,
-                                     config=config,
-                                     # items_history_list=items_history_list,
-                                     stats_init_value={})
+        super(Plugin, self).__init__(
+            args=args,
+            config=config,
+            # items_history_list=items_history_list,
+            stats_init_value={},
+        )
 
         # We want to display the stat in the curse interface
         self.display_curse = True
@@ -131,7 +125,7 @@ class Plugin(GlancesPlugin):
         # Call the father's method
         super(Plugin, self).update_views()
 
-        # Add specifics informations
+        # Add specific information
         try:
             # Alert and log
             if self.nf_conntrack_enabled:
@@ -167,11 +161,10 @@ class Plugin(GlancesPlugin):
             ret.append(self.curse_new_line())
             msg = '{:{width}}'.format(nativestr(s).capitalize(), width=len(s))
             ret.append(self.curse_add_line(msg))
-            msg = '{:>{width}}'.format('{:0.0f}/{:0.0f}'.format(self.stats['nf_conntrack_count'],
-                                                                self.stats['nf_conntrack_max']),
-                                       width=max_width - len(s) + 2)
-            ret.append(self.curse_add_line(msg,
-                                           self.get_views(key='nf_conntrack_percent',
-                                                          option='decoration')))
+            msg = '{:>{width}}'.format(
+                '{:0.0f}/{:0.0f}'.format(self.stats['nf_conntrack_count'], self.stats['nf_conntrack_max']),
+                width=max_width - len(s) + 2,
+            )
+            ret.append(self.curse_add_line(msg, self.get_views(key='nf_conntrack_percent', option='decoration')))
 
         return ret
