@@ -2,20 +2,10 @@
 #
 # This file is part of Glances.
 #
-# Copyright (C) 2019 Nicolargo <nicolas@nicolargo.com>
+# SPDX-FileCopyrightText: 2022 Nicolas Hennion <nicolas@nicolargo.com>
 #
-# Glances is free software; you can redistribute it and/or modify
-# it under the terms of the GNU Lesser General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# SPDX-License-Identifier: LGPL-3.0-only
 #
-# Glances is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public License
-# along with this program. If not, see <http://www.gnu.org/licenses/>.
 
 """Docker plugin."""
 
@@ -24,20 +14,21 @@ import threading
 import time
 from copy import deepcopy
 
+from glances.compat import iterkeys, itervalues, nativestr, pretty_date
 from glances.logger import logger
-from glances.compat import iterkeys, itervalues, nativestr
-from glances.timer import getTimeSinceLastUpdate
 from glances.plugins.glances_plugin import GlancesPlugin
 from glances.processes import sort_stats as sort_stats_processes, glances_processes
+from glances.timer import getTimeSinceLastUpdate
 
 # Docker-py library (optional and Linux-only)
 # https://github.com/docker/docker-py
 try:
     import docker
+    from dateutil import parser, tz
 except Exception as e:
     import_error_tag = True
     # Display debug message if import KeyError
-    logger.warning("Error loading Docker Python Lib. Docker plugin is disabled ({})".format(e))
+    logger.warning("Error loading Docker deps Lib. Docker plugin is disabled ({})".format(e))
 else:
     import_error_tag = False
 
@@ -62,14 +53,21 @@ else:
 #                       {'name': 'io_w',
 #                        'description': 'Container IO bytes write per second',
 #                        'y_unit': 'Bps'}]
-items_history_list = [{'name': 'cpu_percent',
-                       'description': 'Container CPU consumption in %',
-                       'y_unit': '%'}]
-
+items_history_list = [{'name': 'cpu_percent', 'description': 'Container CPU consumption in %', 'y_unit': '%'}]
 
 # List of key to remove before export
 export_exclude_list = ['cpu', 'io', 'memory', 'network']
 
+# Sort dictionary for human
+sort_for_human = {
+    'io_counters': 'disk IO',
+    'cpu_percent': 'CPU consumption',
+    'memory_usage': 'memory consumption',
+    'cpu_times': 'uptime',
+    'name': 'container name',
+    None: 'None',
+}
+
 
 class Plugin(GlancesPlugin):
     """Glances Docker plugin.
@@ -79,9 +77,7 @@ class Plugin(GlancesPlugin):
 
     def __init__(self, args=None, config=None):
         """Init the plugin."""
-        super(Plugin, self).__init__(args=args,
-                                     config=config,
-                                     items_history_list=items_history_list)
+        super(Plugin, self).__init__(args=args, config=config, items_history_list=items_history_list)
 
         # The plugin can be disabled using: args.disable_docker
         self.args = args
@@ -95,11 +91,24 @@ class Plugin(GlancesPlugin):
         # Init the Docker API
         self.docker_client = self.connect()
 
-        # Dict of thread (to grab stats asynchronously, one thread is created by container)
+        # Dict of thread (to grab stats asynchronously, one thread is created per container)
         # key: Container Id
         # value: instance of ThreadDockerGrabber
         self.thread_list = {}
 
+        # Dict of Network stats (Storing previous network stats to compute Rx/s and Tx/s)
+        # key: Container Id
+        # value: network stats dict
+        self.network_old = {}
+
+        # Dict of Disk IO stats (Storing previous disk_io stats to compute Rx/s and Tx/s)
+        # key: Container Id
+        # value: network stats dict
+        self.io_old = {}
+
+        # Sort key
+        self.sort_key = None
+
         # Force a first update because we need two update to have the first stat
         self.update()
         self.refresh_timer.set(0)
@@ -210,15 +219,17 @@ class Plugin(GlancesPlugin):
             for container in containers:
                 if container.id not in self.thread_list:
                     # Thread did not exist in the internal dict
-                    # Create it and add it to the internal dict
-                    logger.debug("{} plugin - Create thread for container {}".format(self.plugin_name, container.id[:12]))
+                    # Create it, add it to the internal dict and start it
+                    logger.debug(
+                        "{} plugin - Create thread for container {}".format(self.plugin_name, container.id[:12])
+                    )
                     t = ThreadDockerGrabber(container)
                     self.thread_list[container.id] = t
                     t.start()
 
             # Stop threads for non-existing containers
-            nonexisting_containers = set(iterkeys(self.thread_list)) - set([c.id for c in containers])
-            for container_id in nonexisting_containers:
+            absent_containers = set(iterkeys(self.thread_list)) - set([c.id for c in containers])
+            for container_id in absent_containers:
                 # Stop the thread
                 logger.debug("{} plugin - Stop thread for old container {}".format(self.plugin_name, container_id[:12]))
                 self.thread_list[container_id].stop()
@@ -228,22 +239,16 @@ class Plugin(GlancesPlugin):
             # Get stats for all containers
             stats['containers'] = []
             for container in containers:
-                # Only show specific containers
-                if not self.is_show(nativestr(container.name)):
-                    continue
-
-                # Do not take hidden container into account
-                if self.is_hide(nativestr(container.name)):
+                # Shall we display the stats ?
+                if not self.is_display(nativestr(container.name)):
                     continue
 
                 # Init the stats for the current container
                 container_stats = {}
                 # The key is the container name and not the Id
                 container_stats['key'] = self.get_key()
-                # Export name (first name in the Names list, without the /)
+                # Export name
                 container_stats['name'] = nativestr(container.name)
-                # Export global Names (used by the WebUI)
-                container_stats['Names'] = [nativestr(container.name)]
                 # Container Id
                 container_stats['Id'] = container.id
                 # Container Image
@@ -267,7 +272,9 @@ class Plugin(GlancesPlugin):
                     container_stats['cpu'] = self.get_docker_cpu(container.id, self.thread_list[container.id].stats)
                     container_stats['cpu_percent'] = container_stats['cpu'].get('total', None)
                     # MEM
-                    container_stats['memory'] = self.get_docker_memory(container.id, self.thread_list[container.id].stats)
+                    container_stats['memory'] = self.get_docker_memory(
+                        container.id, self.thread_list[container.id].stats
+                    )
                     container_stats['memory_usage'] = container_stats['memory'].get('usage', None)
                     if container_stats['memory'].get('cache', None) is not None:
                         container_stats['memory_usage'] -= container_stats['memory']['cache']
@@ -276,9 +283,18 @@ class Plugin(GlancesPlugin):
                     container_stats['io_r'] = container_stats['io'].get('ior', None)
                     container_stats['io_w'] = container_stats['io'].get('iow', None)
                     # NET
-                    container_stats['network'] = self.get_docker_network(container.id, self.thread_list[container.id].stats)
+                    container_stats['network'] = self.get_docker_network(
+                        container.id, self.thread_list[container.id].stats
+                    )
                     container_stats['network_rx'] = container_stats['network'].get('rx', None)
                     container_stats['network_tx'] = container_stats['network'].get('tx', None)
+                    # Uptime
+                    container_stats['Uptime'] = pretty_date(
+                        # parser.parse(container.attrs['State']['StartedAt']).replace(tzinfo=None)
+                        parser.parse(container.attrs['State']['StartedAt'])
+                        .astimezone(tz.tzlocal())
+                        .replace(tzinfo=None)
+                    )
                 else:
                     container_stats['cpu'] = {}
                     container_stats['cpu_percent'] = None
@@ -290,6 +306,7 @@ class Plugin(GlancesPlugin):
                     container_stats['network'] = {}
                     container_stats['network_rx'] = None
                     container_stats['network_tx'] = None
+                    container_stats['Uptime'] = None
                 # Add current container stats to the stats list
                 stats['containers'].append(container_stats)
 
@@ -299,7 +316,7 @@ class Plugin(GlancesPlugin):
             pass
 
         # Sort and update the stats
-        self.stats = sort_stats(stats)
+        self.sort_key, self.stats = sort_docker_stats(stats)
 
         return self.stats
 
@@ -310,51 +327,39 @@ class Plugin(GlancesPlugin):
                all_stats is the output of the stats method of the Docker API
         Output: a dict {'total': 1.49}
         """
-        ret = {'total': 0.0}
+        cpu_stats = {'total': 0.0}
 
-        # Read the stats
-        # For each container, you will find a pseudo-file cpuacct.stat,
-        # containing the CPU usage accumulated by the processes of the container.
-        # Those times are expressed in ticks of 1/USER_HZ of a second.
-        # On x86 systems, USER_HZ is 100.
-        cpu_new = {}
-        precpu_new = {}
         try:
-            cpu_new['total'] = all_stats['cpu_stats']['cpu_usage'].get(
-                'total_usage', None)
-            precpu_new['total'] = all_stats['precpu_stats']['cpu_usage'].get(
-                'total_usage', None)
-            cpu_new['system'] = all_stats['cpu_stats'].get(
-                'system_cpu_usage', None)
-            precpu_new['system'] = all_stats['precpu_stats'].get(
-                'system_cpu_usage', None)
+            cpu = {
+                'system': all_stats['cpu_stats']['system_cpu_usage'],
+                'total': all_stats['cpu_stats']['cpu_usage']['total_usage'],
+            }
+            precpu = {
+                'system': all_stats['precpu_stats']['system_cpu_usage'],
+                'total': all_stats['precpu_stats']['cpu_usage']['total_usage'],
+            }
             # Issue #1857
             # If either precpu_stats.online_cpus or cpu_stats.online_cpus is nil
             # then for compatibility with older daemons the length of
             # the corresponding cpu_usage.percpu_usage array should be used.
-            if 'online_cpus' in all_stats['cpu_stats'] and \
-               all_stats['cpu_stats']['online_cpus'] is not None:
-                cpu_new['nb_core'] = all_stats['cpu_stats']['online_cpus']
-            else:
-                cpu_new['nb_core'] = len(all_stats['cpu_stats']['cpu_usage']['percpu_usage'] or [])
+            cpu['nb_core'] = all_stats['cpu_stats'].get('online_cpus', None)
+            if cpu['nb_core'] is None:
+                cpu['nb_core'] = len(all_stats['cpu_stats']['cpu_usage']['percpu_usage'] or [])
         except KeyError as e:
-            logger.debug(
-                "docker plugin - Cannot grab CPU usage for container {} ({})".format(container_id, e))
+            logger.debug("docker plugin - Cannot grab CPU usage for container {} ({})".format(container_id, e))
             logger.debug(all_stats)
         else:
             try:
-                cpu_delta = cpu_new['total'] - precpu_new['total']
-                system_cpu_delta = cpu_new['system'] - precpu_new['system']
+                cpu_delta = cpu['total'] - precpu['total']
+                system_cpu_delta = cpu['system'] - precpu['system']
                 # CPU usage % = (cpu_delta / system_cpu_delta) * number_cpus * 100.0
-                ret['total'] = (cpu_delta / system_cpu_delta) * \
-                    cpu_new['nb_core'] * 100.0
+                cpu_stats['total'] = (cpu_delta / system_cpu_delta) * cpu['nb_core'] * 100.0
             except TypeError as e:
-                logger.debug(
-                    "docker plugin - Cannot compute CPU usage for container {} ({})".format(container_id, e))
+                logger.debug("docker plugin - Cannot compute CPU usage for container {} ({})".format(container_id, e))
                 logger.debug(all_stats)
 
         # Return the stats
-        return ret
+        return cpu_stats
 
     def get_docker_memory(self, container_id, all_stats):
         """Return the container MEMORY.
@@ -363,30 +368,28 @@ class Plugin(GlancesPlugin):
                all_stats is the output of the stats method of the Docker API
         Output: a dict {'rss': 1015808, 'cache': 356352,  'usage': ..., 'max_usage': ...}
         """
-        ret = {}
+        memory_stats = {}
         # Read the stats
         try:
+            # Mandatory fields
+            memory_stats['usage'] = all_stats['memory_stats']['usage']
+            memory_stats['limit'] = all_stats['memory_stats']['limit']
             # Issue #1857
             # Some stats are not always available in ['memory_stats']['stats']
             if 'rss' in all_stats['memory_stats']['stats']:
-                ret['rss'] = all_stats['memory_stats']['stats']['rss']
+                memory_stats['rss'] = all_stats['memory_stats']['stats']['rss']
             elif 'total_rss' in all_stats['memory_stats']['stats']:
-                ret['rss'] = all_stats['memory_stats']['stats']['total_rss']
+                memory_stats['rss'] = all_stats['memory_stats']['stats']['total_rss']
             else:
-                ret['rss'] = None
-            ret['cache'] = all_stats['memory_stats']['stats'].get(
-                'cache', None)
-            ret['max_usage'] = all_stats['memory_stats'].get(
-                'max_usage', None)
-            # Mandatory fields
-            ret['usage'] = all_stats['memory_stats']['usage']
-            ret['limit'] = all_stats['memory_stats']['limit']
+                memory_stats['rss'] = None
+            memory_stats['cache'] = all_stats['memory_stats']['stats'].get('cache', None)
+            memory_stats['max_usage'] = all_stats['memory_stats'].get('max_usage', None)
         except (KeyError, TypeError) as e:
             # all_stats do not have MEM information
             logger.debug("docker plugin - Cannot grab MEM usage for container {} ({})".format(container_id, e))
             logger.debug(all_stats)
         # Return the stats
-        return ret
+        return memory_stats
 
     def get_docker_network(self, container_id, all_stats):
         """Return the container network usage using the Docker API (v1.0 or higher).
@@ -403,7 +406,7 @@ class Plugin(GlancesPlugin):
 
         # Read the rx/tx stats (in bytes)
         try:
-            netcounters = all_stats["networks"]
+            net_stats = all_stats["networks"]
         except KeyError as e:
             # all_stats do not have NETWORK information
             logger.debug("docker plugin - Cannot grab NET usage for container {} ({})".format(container_id, e))
@@ -411,37 +414,26 @@ class Plugin(GlancesPlugin):
             # No fallback available...
             return network_new
 
-        # Previous network interface stats are stored in the network_old variable
-        if not hasattr(self, 'netcounters_old'):
-            # First call, we init the network_old var
-            self.netcounters_old = {}
-            try:
-                self.netcounters_old[container_id] = netcounters
-            except (IOError, UnboundLocalError):
-                pass
-
-        if container_id not in self.netcounters_old:
-            try:
-                self.netcounters_old[container_id] = netcounters
-            except (IOError, UnboundLocalError):
-                pass
+        # Previous network interface stats are stored in the self.network_old variable
+        # By storing time data we enable Rx/s and Tx/s calculations in the XML/RPC API, which would otherwise
+        # be overly difficult work for users of the API
+        try:
+            network_new['cumulative_rx'] = net_stats["eth0"]["rx_bytes"]
+            network_new['cumulative_tx'] = net_stats["eth0"]["tx_bytes"]
+        except KeyError as e:
+            # all_stats do not have INTERFACE information
+            logger.debug(
+                "docker plugin - Cannot grab network interface usage for container {} ({})".format(container_id, e)
+            )
+            logger.debug(all_stats)
         else:
-            # By storing time data we enable Rx/s and Tx/s calculations in the
-            # XML/RPC API, which would otherwise be overly difficult work
-            # for users of the API
-            try:
-                network_new['time_since_update'] = getTimeSinceLastUpdate('docker_net_{}'.format(container_id))
-                network_new['rx'] = netcounters["eth0"]["rx_bytes"] - self.netcounters_old[container_id]["eth0"]["rx_bytes"]
-                network_new['tx'] = netcounters["eth0"]["tx_bytes"] - self.netcounters_old[container_id]["eth0"]["tx_bytes"]
-                network_new['cumulative_rx'] = netcounters["eth0"]["rx_bytes"]
-                network_new['cumulative_tx'] = netcounters["eth0"]["tx_bytes"]
-            except KeyError as e:
-                # all_stats do not have INTERFACE information
-                logger.debug("docker plugin - Cannot grab network interface usage for container {} ({})".format(container_id, e))
-                logger.debug(all_stats)
+            network_new['time_since_update'] = getTimeSinceLastUpdate('docker_net_{}'.format(container_id))
+            if container_id in self.network_old:
+                network_new['rx'] = network_new['cumulative_rx'] - self.network_old[container_id]['cumulative_rx']
+                network_new['tx'] = network_new['cumulative_tx'] - self.network_old[container_id]['cumulative_tx']
 
             # Save stats to compute next bitrate
-            self.netcounters_old[container_id] = netcounters
+            self.network_old[container_id] = network_new
 
         # Return the stats
         return network_new
@@ -461,7 +453,7 @@ class Plugin(GlancesPlugin):
 
         # Read the ior/iow stats (in bytes)
         try:
-            iocounters = all_stats["blkio_stats"]
+            io_stats = all_stats["blkio_stats"]
         except KeyError as e:
             # all_stats do not have io information
             logger.debug("docker plugin - Cannot grab block IO usage for container {} ({})".format(container_id, e))
@@ -469,45 +461,27 @@ class Plugin(GlancesPlugin):
             # No fallback available...
             return io_new
 
-        # Previous io interface stats are stored in the io_old variable
-        if not hasattr(self, 'iocounters_old'):
-            # First call, we init the io_old var
-            self.iocounters_old = {}
-            try:
-                self.iocounters_old[container_id] = iocounters
-            except (IOError, UnboundLocalError):
-                pass
+        # Previous io interface stats are stored in the self.io_old variable
+        # By storing time data we enable IoR/s and IoW/s calculations in the
+        # XML/RPC API, which would otherwise be overly difficult work
+        # for users of the API
+        try:
+            io_service_bytes_recursive = io_stats['io_service_bytes_recursive']
 
-        if container_id not in self.iocounters_old:
-            try:
-                self.iocounters_old[container_id] = iocounters
-            except (IOError, UnboundLocalError):
-                pass
+            # Read IOR and IOW value in the structure list of dict
+            io_new['cumulative_ior'] = [i for i in io_service_bytes_recursive if i['op'].lower() == 'read'][0]['value']
+            io_new['cumulative_iow'] = [i for i in io_service_bytes_recursive if i['op'].lower() == 'write'][0]['value']
+        except (TypeError, IndexError, KeyError, AttributeError) as e:
+            # all_stats do not have io information
+            logger.debug("docker plugin - Cannot grab block IO usage for container {} ({})".format(container_id, e))
         else:
-            # By storing time data we enable IoR/s and IoW/s calculations in the
-            # XML/RPC API, which would otherwise be overly difficult work
-            # for users of the API
-            try:
-                new_io_service_bytes_recursive = iocounters['io_service_bytes_recursive']
-                old_io_service_bytes_recursive = self.iocounters_old[container_id]['io_service_bytes_recursive']
+            io_new['time_since_update'] = getTimeSinceLastUpdate('docker_io_{}'.format(container_id))
+            if container_id in self.io_old:
+                io_new['ior'] = io_new['cumulative_ior'] - self.io_old[container_id]['cumulative_ior']
+                io_new['iow'] = io_new['cumulative_iow'] - self.io_old[container_id]['cumulative_iow']
 
-                # Read IOR and IOW value in the structure list of dict
-                ior = [i for i in new_io_service_bytes_recursive if i['op'].lower() == 'read'][0]['value']
-                iow = [i for i in new_io_service_bytes_recursive if i['op'].lower() == 'write'][0]['value']
-                ior_old = [i for i in old_io_service_bytes_recursive if i['op'].lower() == 'read'][0]['value']
-                iow_old = [i for i in old_io_service_bytes_recursive if i['op'].lower() == 'write'][0]['value']
-            except (TypeError, IndexError, KeyError, AttributeError) as e:
-                # all_stats do not have io information
-                logger.debug("docker plugin - Cannot grab block IO usage for container {} ({})".format(container_id, e))
-            else:
-                io_new['time_since_update'] = getTimeSinceLastUpdate('docker_io_{}'.format(container_id))
-                io_new['ior'] = ior - ior_old
-                io_new['iow'] = iow - iow_old
-                io_new['cumulative_ior'] = ior
-                io_new['cumulative_iow'] = iow
-
-                # Save stats to compute next bitrate
-                self.iocounters_old[container_id] = iocounters
+            # Save stats to compute next bitrate
+            self.io_old[container_id] = io_new
 
         # Return the stats
         return io_new
@@ -539,9 +513,7 @@ class Plugin(GlancesPlugin):
             # CPU alert
             if 'cpu' in i and 'total' in i['cpu']:
                 # Looking for specific CPU container threshold in the conf file
-                alert = self.get_alert(i['cpu']['total'],
-                                       header=i['name'] + '_cpu',
-                                       action_key=i['name'])
+                alert = self.get_alert(i['cpu']['total'], header=i['name'] + '_cpu', action_key=i['name'])
                 if alert == 'DEFAULT':
                     # Not found ? Get back to default CPU threshold value
                     alert = self.get_alert(i['cpu']['total'], header='cpu')
@@ -549,15 +521,12 @@ class Plugin(GlancesPlugin):
             # MEM alert
             if 'memory' in i and 'usage' in i['memory']:
                 # Looking for specific MEM container threshold in the conf file
-                alert = self.get_alert(i['memory']['usage'],
-                                       maximum=i['memory']['limit'],
-                                       header=i['name'] + '_mem',
-                                       action_key=i['name'])
+                alert = self.get_alert(
+                    i['memory']['usage'], maximum=i['memory']['limit'], header=i['name'] + '_mem', action_key=i['name']
+                )
                 if alert == 'DEFAULT':
                     # Not found ? Get back to default MEM threshold value
-                    alert = self.get_alert(i['memory']['usage'],
-                                           maximum=i['memory']['limit'],
-                                           header='mem')
+                    alert = self.get_alert(i['memory']['usage'], maximum=i['memory']['limit'], header='mem')
                 self.views[i[self.get_key()]]['mem']['decoration'] = alert
 
         return True
@@ -568,9 +537,7 @@ class Plugin(GlancesPlugin):
         ret = []
 
         # Only process if stats exist (and non null) and display plugin enable...
-        if not self.stats \
-           or 'containers' not in self.stats or len(self.stats['containers']) == 0 \
-           or self.is_disabled():
+        if not self.stats or 'containers' not in self.stats or len(self.stats['containers']) == 0 or self.is_disabled():
             return ret
 
         # Build the string message
@@ -579,36 +546,38 @@ class Plugin(GlancesPlugin):
         ret.append(self.curse_add_line(msg, "TITLE"))
         msg = ' {}'.format(len(self.stats['containers']))
         ret.append(self.curse_add_line(msg))
-        msg = ' (served by Docker {})'.format(self.stats['version']["Version"])
+        msg = ' sorted by {}'.format(sort_for_human[self.sort_key])
         ret.append(self.curse_add_line(msg))
+        # msg = ' (served by Docker {})'.format(self.stats['version']["Version"])
+        # ret.append(self.curse_add_line(msg))
         ret.append(self.curse_new_line())
         # Header
         ret.append(self.curse_new_line())
         # Get the maximum containers name
         # Max size is configurable. See feature request #1723.
-        name_max_width = min(self.config.get_int_value('docker',
-                                                       'max_name_size',
-                                                       default=20)
-                             if self.config is not None else 20,
-                             len(max(self.stats['containers'],
-                                 key=lambda x: len(x['name']))['name']))
+        name_max_width = min(
+            self.config.get_int_value('docker', 'max_name_size', default=20) if self.config is not None else 20,
+            len(max(self.stats['containers'], key=lambda x: len(x['name']))['name']),
+        )
         msg = ' {:{width}}'.format('Name', width=name_max_width)
-        ret.append(self.curse_add_line(msg))
+        ret.append(self.curse_add_line(msg, 'SORT' if self.sort_key == 'name' else 'DEFAULT'))
         msg = '{:>10}'.format('Status')
         ret.append(self.curse_add_line(msg))
+        msg = '{:>10}'.format('Uptime')
+        ret.append(self.curse_add_line(msg))
         msg = '{:>6}'.format('CPU%')
-        ret.append(self.curse_add_line(msg))
+        ret.append(self.curse_add_line(msg, 'SORT' if self.sort_key == 'cpu_percent' else 'DEFAULT'))
         msg = '{:>7}'.format('MEM')
-        ret.append(self.curse_add_line(msg))
-        msg = '{:>7}'.format('/MAX')
+        ret.append(self.curse_add_line(msg, 'SORT' if self.sort_key == 'memory_usage' else 'DEFAULT'))
+        msg = '/{:<7}'.format('MAX')
         ret.append(self.curse_add_line(msg))
         msg = '{:>7}'.format('IOR/s')
         ret.append(self.curse_add_line(msg))
-        msg = '{:>7}'.format('IOW/s')
+        msg = ' {:<7}'.format('IOW/s')
         ret.append(self.curse_add_line(msg))
         msg = '{:>7}'.format('Rx/s')
         ret.append(self.curse_add_line(msg))
-        msg = '{:>7}'.format('Tx/s')
+        msg = ' {:<7}'.format('Tx/s')
         ret.append(self.curse_add_line(msg))
         msg = ' {:8}'.format('Command')
         ret.append(self.curse_add_line(msg))
@@ -616,42 +585,48 @@ class Plugin(GlancesPlugin):
         for container in self.stats['containers']:
             ret.append(self.curse_new_line())
             # Name
-            ret.append(self.curse_add_line(self._msg_name(container=container,
-                                                          max_width=name_max_width)))
+            ret.append(self.curse_add_line(self._msg_name(container=container, max_width=name_max_width)))
             # Status
             status = self.container_alert(container['Status'])
             msg = '{:>10}'.format(container['Status'][0:10])
             ret.append(self.curse_add_line(msg, status))
+            # Uptime
+            if container['Uptime']:
+                msg = '{:>10}'.format(container['Uptime'])
+            else:
+                msg = '{:>10}'.format('_')
+            ret.append(self.curse_add_line(msg, status))
             # CPU
             try:
                 msg = '{:>6.1f}'.format(container['cpu']['total'])
             except KeyError:
                 msg = '{:>6}'.format('_')
-            ret.append(self.curse_add_line(msg, self.get_views(item=container['name'],
-                                                               key='cpu',
-                                                               option='decoration')))
+            ret.append(self.curse_add_line(msg, self.get_views(item=container['name'], key='cpu', option='decoration')))
             # MEM
             try:
                 msg = '{:>7}'.format(self.auto_unit(container['memory']['usage']))
             except KeyError:
                 msg = '{:>7}'.format('_')
-            ret.append(self.curse_add_line(msg, self.get_views(item=container['name'],
-                                                               key='mem',
-                                                               option='decoration')))
+            ret.append(self.curse_add_line(msg, self.get_views(item=container['name'], key='mem', option='decoration')))
             try:
-                msg = '{:>7}'.format(self.auto_unit(container['memory']['limit']))
+                msg = '/{:<7}'.format(self.auto_unit(container['memory']['limit']))
             except KeyError:
-                msg = '{:>7}'.format('_')
+                msg = '/{:<7}'.format('_')
             ret.append(self.curse_add_line(msg))
             # IO R/W
             unit = 'B'
-            for r in ['ior', 'iow']:
-                try:
-                    value = self.auto_unit(int(container['io'][r] // container['io']['time_since_update'])) + unit
-                    msg = '{:>7}'.format(value)
-                except KeyError:
-                    msg = '{:>7}'.format('_')
-                ret.append(self.curse_add_line(msg))
+            try:
+                value = self.auto_unit(int(container['io']['ior'] // container['io']['time_since_update'])) + unit
+                msg = '{:>7}'.format(value)
+            except KeyError:
+                msg = '{:>7}'.format('_')
+            ret.append(self.curse_add_line(msg))
+            try:
+                value = self.auto_unit(int(container['io']['iow'] // container['io']['time_since_update'])) + unit
+                msg = ' {:<7}'.format(value)
+            except KeyError:
+                msg = ' {:<7}'.format('_')
+            ret.append(self.curse_add_line(msg))
             # NET RX/TX
             if args.byte:
                 # Bytes per second (for dummy)
@@ -661,13 +636,28 @@ class Plugin(GlancesPlugin):
                 # Bits per second (for real network administrator | Default)
                 to_bit = 8
                 unit = 'b'
-            for r in ['rx', 'tx']:
-                try:
-                    value = self.auto_unit(int(container['network'][r] // container['network']['time_since_update'] * to_bit)) + unit
-                    msg = '{:>7}'.format(value)
-                except KeyError:
-                    msg = '{:>7}'.format('_')
-                ret.append(self.curse_add_line(msg))
+            try:
+                value = (
+                    self.auto_unit(
+                        int(container['network']['rx'] // container['network']['time_since_update'] * to_bit)
+                    )
+                    + unit
+                )
+                msg = '{:>7}'.format(value)
+            except KeyError:
+                msg = '{:>7}'.format('_')
+            ret.append(self.curse_add_line(msg))
+            try:
+                value = (
+                    self.auto_unit(
+                        int(container['network']['tx'] // container['network']['time_since_update'] * to_bit)
+                    )
+                    + unit
+                )
+                msg = ' {:<7}'.format(value)
+            except KeyError:
+                msg = ' {:<7}'.format('_')
+            ret.append(self.curse_add_line(msg))
             # Command
             if container['Command'] is not None:
                 msg = ' {}'.format(' '.join(container['Command']))
@@ -679,11 +669,7 @@ class Plugin(GlancesPlugin):
 
     def _msg_name(self, container, max_width):
         """Build the container name."""
-        name = container['name']
-        if len(name) > max_width:
-            name = '_' + name[-max_width + 1:]
-        else:
-            name = name[:max_width]
+        name = container['name'][:max_width]
         return ' {:{width}}'.format(name, width=max_width)
 
     def container_alert(self, status):
@@ -731,8 +717,8 @@ class ThreadDockerGrabber(threading.Thread):
                 time.sleep(0.1)
                 if self.stopped():
                     break
-        except:
-            logger.debug("docker plugin - Exception thrown during run")
+        except Exception as e:
+            logger.debug("docker plugin - Exception thrown during run ({})".format(e))
             self.stop()
 
     @property
@@ -752,17 +738,27 @@ class ThreadDockerGrabber(threading.Thread):
 
     def stopped(self):
         """Return True is the thread is stopped."""
-        return self._stopper.isSet()
+        return self._stopper.is_set()
 
 
-def sort_stats(stats):
+def sort_docker_stats(stats):
     # Sort Docker stats using the same function than processes
-    sort_by = 'cpu_percent'
+    sort_by = glances_processes.sort_key
     sort_by_secondary = 'memory_usage'
-    if glances_processes.sort_key.startswith('memory'):
+    if sort_by == 'memory_percent':
         sort_by = 'memory_usage'
         sort_by_secondary = 'cpu_percent'
-    sort_stats_processes(stats['containers'],
-                         sortedby=sort_by,
-                         sortedby_secondary=sort_by_secondary)
-    return stats
+    elif sort_by in ['username', 'io_counters', 'cpu_times']:
+        sort_by = 'cpu_percent'
+
+    # Sort docker stats
+    sort_stats_processes(
+        stats['containers'],
+        sorted_by=sort_by,
+        sorted_by_secondary=sort_by_secondary,
+        # Reverse for all but name
+        reverse=glances_processes.sort_key != 'name',
+    )
+
+    # Return the main sort key and the sorted stats
+    return sort_by, stats
