@@ -2,39 +2,23 @@
 #
 # This file is part of Glances.
 #
-# Copyright (C) 2019 Nicolargo <nicolas@nicolargo.com>
+# SPDX-FileCopyrightText: 2022 Nicolas Hennion <nicolas@nicolargo.com>
 #
-# Glances is free software; you can redistribute it and/or modify
-# it under the terms of the GNU Lesser General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# SPDX-License-Identifier: LGPL-3.0-only
 #
-# Glances is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public License
-# along with this program. If not, see <http://www.gnu.org/licenses/>.
 
 """Process count plugin."""
 
-from glances.processes import glances_processes
+from glances.processes import glances_processes, sort_for_human
 from glances.plugins.glances_plugin import GlancesPlugin
 
 # Define the history items list
-items_history_list = [{'name': 'total',
-                       'description': 'Total number of processes',
-                       'y_unit': ''},
-                      {'name': 'running',
-                       'description': 'Total number of running processes',
-                       'y_unit': ''},
-                      {'name': 'sleeping',
-                       'description': 'Total number of sleeping processes',
-                       'y_unit': ''},
-                      {'name': 'thread',
-                       'description': 'Total number of threads',
-                       'y_unit': ''}]
+items_history_list = [
+    {'name': 'total', 'description': 'Total number of processes', 'y_unit': ''},
+    {'name': 'running', 'description': 'Total number of running processes', 'y_unit': ''},
+    {'name': 'sleeping', 'description': 'Total number of sleeping processes', 'y_unit': ''},
+    {'name': 'thread', 'description': 'Total number of threads', 'y_unit': ''},
+]
 
 
 class Plugin(GlancesPlugin):
@@ -43,23 +27,25 @@ class Plugin(GlancesPlugin):
     stats is a list
     """
 
-    sort_for_human = {'io_counters': 'disk IO',
-                      'cpu_percent': 'CPU consumption',
-                      'memory_percent': 'memory consumption',
-                      'cpu_times': 'process time',
-                      'name': 'process name',
-                      None: 'None'}
-
-    def __init__(self, args=None):
+    def __init__(self, args=None, config=None):
         """Init the plugin."""
-        super(Plugin, self).__init__(args=args,
-                                     items_history_list=items_history_list)
+        super(Plugin, self).__init__(args=args, config=config, items_history_list=items_history_list)
 
         # We want to display the stat in the curse interface
         self.display_curse = True
 
         # Note: 'glances_processes' is already init in the glances_processes.py script
 
+    def enable_extended(self):
+        """Enable extended stats."""
+        glances_processes.enable_extended()
+
+    def disable_extended(self):
+        """Disable extended stats."""
+        glances_processes.disable_extended()
+
+    @GlancesPlugin._check_decorator
+    @GlancesPlugin._log_result_decorator
     def update(self):
         """Update processes stats using the input method."""
         # Init new stats
@@ -71,10 +57,10 @@ class Plugin(GlancesPlugin):
             glances_processes.update()
 
             # Return the processes count
-            stats = glances_processes.getcount()
+            stats = glances_processes.get_count()
         elif self.input_method == 'snmp':
             # Update stats using SNMP
-            # Not avalaible
+            # Not available
             pass
 
         # Update the stats
@@ -135,16 +121,17 @@ class Plugin(GlancesPlugin):
         ret.append(self.curse_add_line(msg))
 
         # Display sort information
+        msg = 'Programs' if self.args.programs else 'Threads'
         try:
-            sort_human = self.sort_for_human[glances_processes.sort_key]
+            sort_human = sort_for_human[glances_processes.sort_key]
         except KeyError:
-            sort_human = '?'
+            sort_human = glances_processes.sort_key
         if glances_processes.auto_sort:
-            msg = 'sorted automatically'
+            msg += ' sorted automatically'
             ret.append(self.curse_add_line(msg))
             msg = ' by {}'.format(sort_human)
         else:
-            msg = 'sorted by {}'.format(sort_human)
+            msg += ' sorted by {}'.format(sort_human)
         ret.append(self.curse_add_line(msg))
 
         # Return the message with decoration
