@@ -2,33 +2,24 @@
 #
 # This file is part of Glances.
 #
-# Copyright (C) 2019 Nicolargo <nicolas@nicolargo.com>
+# SPDX-FileCopyrightText: 2022 Nicolas Hennion <nicolas@nicolargo.com>
 #
-# Glances is free software; you can redistribute it and/or modify
-# it under the terms of the GNU Lesser General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# SPDX-License-Identifier: LGPL-3.0-only
 #
-# Glances is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public License
-# along with this program. If not, see <http://www.gnu.org/licenses/>.
 
 """Process list plugin."""
 
 import os
-import shlex
 import copy
-from datetime import timedelta
 
 from glances.logger import logger
 from glances.globals import WINDOWS
+from glances.compat import key_exist_value_not_none_not_v
 from glances.processes import glances_processes, sort_stats
+from glances.outputs.glances_unicode import unicode_message
 from glances.plugins.glances_core import Plugin as CorePlugin
 from glances.plugins.glances_plugin import GlancesPlugin
+from glances.programs import processes_to_programs
 
 
 def seconds_to_hms(input_seconds):
@@ -43,10 +34,21 @@ def seconds_to_hms(input_seconds):
     return hours, minutes, seconds
 
 
-def split_cmdline(cmdline):
-    """Return path, cmd and arguments for a process cmdline."""
-    path, cmd = os.path.split(cmdline[0])
-    arguments = ' '.join(cmdline[1:])
+def split_cmdline(bare_process_name, cmdline):
+    """Return path, cmd and arguments for a process cmdline based on bare_process_name.
+
+    If first argument of cmdline starts with the bare_process_name then
+    cmdline will just be considered cmd and path will be empty (see https://github.com/nicolargo/glances/issues/1795)
+
+    :param bare_process_name: Name of the process from psutil
+    :param cmdline: cmdline from psutil
+    :return: Tuple with three strings, which are path, cmd and arguments of the process
+    """
+    if cmdline[0].startswith(bare_process_name):
+        path, cmd = "", cmdline[0]
+    else:
+        path, cmd = os.path.split(cmdline[0])
+    arguments = ' '.join(cmdline[1:]).replace('\n', ' ')
     return path, cmd, arguments
 
 
@@ -70,12 +72,13 @@ class Plugin(GlancesPlugin):
         'status': '{:>1} ',
         'ior': '{:>4} ',
         'iow': '{:<4} ',
-        'command': '{}',
+        'command': '{} {}',
     }
 
     # Define the stat layout of the processes list columns
     layout_stat = {
-        'cpu': '{:<6.1f} ',
+        'cpu': '{:<6.1f}',
+        'cpu_no_digit': '{:<6.0f}',
         'mem': '{:<5.1f} ',
         'virt': '{:<5} ',
         'res': '{:<5} ',
@@ -88,14 +91,12 @@ class Plugin(GlancesPlugin):
         'ior': '{:>4} ',
         'iow': '{:<4} ',
         'command': '{}',
-        'name': '[{}]'
+        'name': '[{}]',
     }
 
     def __init__(self, args=None, config=None):
         """Init the plugin."""
-        super(Plugin, self).__init__(args=args,
-                                     config=config,
-                                     stats_init_value=[])
+        super(Plugin, self).__init__(args=args, config=config, stats_init_value=[])
 
         # We want to display the stat in the curse interface
         self.display_curse = True
@@ -116,6 +117,20 @@ class Plugin(GlancesPlugin):
         # Use to optimize space (see https://github.com/nicolargo/glances/issues/959)
         self.pid_max = glances_processes.pid_max
 
+        # Set the default sort key if it is defined in the configuration file
+        if config is not None:
+            if 'processlist' in config.as_dict() and 'sort_key' in config.as_dict()['processlist']:
+                logger.debug(
+                    'Configuration overwrites processes sort key by {}'.format(
+                        config.as_dict()['processlist']['sort_key']
+                    )
+                )
+                glances_processes.set_sort_key(config.as_dict()['processlist']['sort_key'], False)
+
+        # The default sort key could also be overwrite by command line (see #1903)
+        if args.sort_processes_key is not None:
+            glances_processes.set_sort_key(args.sort_processes_key, False)
+
         # Note: 'glances_processes' is already init in the processes.py script
 
     def get_key(self):
@@ -132,6 +147,8 @@ class Plugin(GlancesPlugin):
             # Note: Update is done in the processcount plugin
             # Just return the processes list
             stats = glances_processes.getlist()
+            if self.args.programs:
+                stats = processes_to_programs(stats)
 
         elif self.input_method == 'snmp':
             # No SNMP grab for processes
@@ -166,75 +183,87 @@ class Plugin(GlancesPlugin):
             pass
         return 'DEFAULT'
 
-    def get_process_curses_data(self, p, first, args):
-        """Get curses data to display for a process.
-
-        - p is the process to display
-        - first is a tag=True if the process is the first on the list
-        """
-        ret = [self.curse_new_line()]
-        # CPU
-        if 'cpu_percent' in p and p['cpu_percent'] is not None and p['cpu_percent'] != '':
+    def _get_process_curses_cpu(self, p, selected, args):
+        """Return process CPU curses"""
+        if key_exist_value_not_none_not_v('cpu_percent', p, ''):
+            cpu_layout = self.layout_stat['cpu'] if p['cpu_percent'] < 100 else self.layout_stat['cpu_no_digit']
             if args.disable_irix and self.nb_log_core != 0:
-                msg = self.layout_stat['cpu'].format(p['cpu_percent'] / float(self.nb_log_core))
+                msg = cpu_layout.format(p['cpu_percent'] / float(self.nb_log_core))
             else:
-                msg = self.layout_stat['cpu'].format(p['cpu_percent'])
-            alert = self.get_alert(p['cpu_percent'],
-                                   highlight_zero=False,
-                                   is_max=(p['cpu_percent'] == self.max_values['cpu_percent']),
-                                   header="cpu")
-            ret.append(self.curse_add_line(msg, alert))
+                msg = cpu_layout.format(p['cpu_percent'])
+            alert = self.get_alert(
+                p['cpu_percent'],
+                highlight_zero=False,
+                is_max=(p['cpu_percent'] == self.max_values['cpu_percent']),
+                header="cpu",
+            )
+            ret = self.curse_add_line(msg, alert)
         else:
             msg = self.layout_header['cpu'].format('?')
-            ret.append(self.curse_add_line(msg))
-        # MEM
-        if 'memory_percent' in p and p['memory_percent'] is not None and p['memory_percent'] != '':
+            ret = self.curse_add_line(msg)
+        return ret
+
+    def _get_process_curses_mem(self, p, selected, args):
+        """Return process MEM curses"""
+        if key_exist_value_not_none_not_v('memory_percent', p, ''):
             msg = self.layout_stat['mem'].format(p['memory_percent'])
-            alert = self.get_alert(p['memory_percent'],
-                                   highlight_zero=False,
-                                   is_max=(p['memory_percent'] == self.max_values['memory_percent']),
-                                   header="mem")
-            ret.append(self.curse_add_line(msg, alert))
+            alert = self.get_alert(
+                p['memory_percent'],
+                highlight_zero=False,
+                is_max=(p['memory_percent'] == self.max_values['memory_percent']),
+                header="mem",
+            )
+            ret = self.curse_add_line(msg, alert)
         else:
             msg = self.layout_header['mem'].format('?')
-            ret.append(self.curse_add_line(msg))
-        # VMS/RSS
-        if 'memory_info' in p and p['memory_info'] is not None and p['memory_info'] != '':
-            # VMS
+            ret = self.curse_add_line(msg)
+        return ret
+
+    def _get_process_curses_vms(self, p, selected, args):
+        """Return process VMS curses"""
+        if key_exist_value_not_none_not_v('memory_info', p, ''):
             msg = self.layout_stat['virt'].format(self.auto_unit(p['memory_info'][1], low_precision=False))
-            ret.append(self.curse_add_line(msg, optional=True))
-            # RSS
-            msg = self.layout_stat['res'].format(self.auto_unit(p['memory_info'][0], low_precision=False))
-            ret.append(self.curse_add_line(msg, optional=True))
+            ret = self.curse_add_line(msg, optional=True)
         else:
             msg = self.layout_header['virt'].format('?')
-            ret.append(self.curse_add_line(msg))
+            ret = self.curse_add_line(msg)
+        return ret
+
+    def _get_process_curses_rss(self, p, selected, args):
+        """Return process RSS curses"""
+        if key_exist_value_not_none_not_v('memory_info', p, ''):
+            msg = self.layout_stat['res'].format(self.auto_unit(p['memory_info'][0], low_precision=False))
+            ret = self.curse_add_line(msg, optional=True)
+        else:
             msg = self.layout_header['res'].format('?')
-            ret.append(self.curse_add_line(msg))
-        # PID
-        msg = self.layout_stat['pid'].format(p['pid'], width=self.__max_pid_size())
-        ret.append(self.curse_add_line(msg))
-        # USER
+            ret = self.curse_add_line(msg)
+        return ret
+
+    def _get_process_curses_username(self, p, selected, args):
+        """Return process username curses"""
         if 'username' in p:
             # docker internal users are displayed as ints only, therefore str()
             # Correct issue #886 on Windows OS
             msg = self.layout_stat['user'].format(str(p['username'])[:9])
-            ret.append(self.curse_add_line(msg))
+            ret = self.curse_add_line(msg)
         else:
             msg = self.layout_header['user'].format('?')
-            ret.append(self.curse_add_line(msg))
-        # TIME+
+            ret = self.curse_add_line(msg)
+        return ret
+
+    def _get_process_curses_time(self, p, selected, args):
+        """Return process time curses"""
         try:
             # Sum user and system time
             user_system_time = p['cpu_times'][0] + p['cpu_times'][1]
-        except (OverflowError, TypeError) as e:
+        except (OverflowError, TypeError):
             # Catch OverflowError on some Amazon EC2 server
             # See https://github.com/nicolargo/glances/issues/87
             # Also catch TypeError on macOS
             # See: https://github.com/nicolargo/glances/issues/622
             # logger.debug("Cannot get TIME+ ({})".format(e))
             msg = self.layout_header['time'].format('?')
-            ret.append(self.curse_add_line(msg, optional=True))
+            ret = self.curse_add_line(msg, optional=True)
         else:
             hours, minutes, seconds = seconds_to_hms(user_system_time)
             if hours > 99:
@@ -245,98 +274,162 @@ class Plugin(GlancesPlugin):
                 msg = '{}:{}'.format(minutes, seconds)
             msg = self.layout_stat['time'].format(msg)
             if hours > 0:
-                ret.append(self.curse_add_line(msg,
-                                               decoration='CPU_TIME',
-                                               optional=True))
+                ret = self.curse_add_line(msg, decoration='CPU_TIME', optional=True)
             else:
-                ret.append(self.curse_add_line(msg, optional=True))
-        # THREAD
+                ret = self.curse_add_line(msg, optional=True)
+        return ret
+
+    def _get_process_curses_thread(self, p, selected, args):
+        """Return process thread curses"""
         if 'num_threads' in p:
             num_threads = p['num_threads']
             if num_threads is None:
                 num_threads = '?'
             msg = self.layout_stat['thread'].format(num_threads)
-            ret.append(self.curse_add_line(msg))
+            ret = self.curse_add_line(msg)
         else:
             msg = self.layout_header['thread'].format('?')
-            ret.append(self.curse_add_line(msg))
-        # NICE
+            ret = self.curse_add_line(msg)
+        return ret
+
+    def _get_process_curses_nice(self, p, selected, args):
+        """Return process nice curses"""
         if 'nice' in p:
             nice = p['nice']
             if nice is None:
                 nice = '?'
             msg = self.layout_stat['nice'].format(nice)
-            ret.append(self.curse_add_line(msg,
-                                           decoration=self.get_nice_alert(nice)))
+            ret = self.curse_add_line(msg, decoration=self.get_nice_alert(nice))
         else:
             msg = self.layout_header['nice'].format('?')
-            ret.append(self.curse_add_line(msg))
-        # STATUS
+            ret = self.curse_add_line(msg)
+        return ret
+
+    def _get_process_curses_status(self, p, selected, args):
+        """Return process status curses"""
         if 'status' in p:
             status = p['status']
             msg = self.layout_stat['status'].format(status)
             if status == 'R':
-                ret.append(self.curse_add_line(msg, decoration='STATUS'))
+                ret = self.curse_add_line(msg, decoration='STATUS')
             else:
-                ret.append(self.curse_add_line(msg))
+                ret = self.curse_add_line(msg)
         else:
             msg = self.layout_header['status'].format('?')
-            ret.append(self.curse_add_line(msg))
-        # IO read/write
+            ret = self.curse_add_line(msg)
+        return ret
+
+    def _get_process_curses_io(self, p, selected, args, rorw='ior'):
+        """Return process IO Read or Write curses"""
         if 'io_counters' in p and p['io_counters'][4] == 1 and p['time_since_update'] != 0:
             # Display rate if stats is available and io_tag ([4]) == 1
-            # IO read
-            io_rs = int((p['io_counters'][0] - p['io_counters'][2]) / p['time_since_update'])
-            if io_rs == 0:
-                msg = self.layout_stat['ior'].format("0")
+            # IO
+            io = int(
+                (p['io_counters'][0 if rorw == 'ior' else 1] - p['io_counters'][2 if rorw == 'ior' else 3])
+                / p['time_since_update']
+            )
+            if io == 0:
+                msg = self.layout_stat[rorw].format("0")
             else:
-                msg = self.layout_stat['ior'].format(self.auto_unit(io_rs,
-                                                                    low_precision=True))
-            ret.append(self.curse_add_line(msg, optional=True, additional=True))
-            # IO write
-            io_ws = int((p['io_counters'][1] - p['io_counters'][3]) / p['time_since_update'])
-            if io_ws == 0:
-                msg = self.layout_stat['iow'].format("0")
-            else:
-                msg = self.layout_stat['iow'].format(self.auto_unit(io_ws,
-                                                                    low_precision=True))
-            ret.append(self.curse_add_line(msg, optional=True, additional=True))
+                msg = self.layout_stat[rorw].format(self.auto_unit(io, low_precision=True))
+            ret = self.curse_add_line(msg, optional=True, additional=True)
         else:
-            msg = self.layout_header['ior'].format("?")
-            ret.append(self.curse_add_line(msg, optional=True, additional=True))
-            msg = self.layout_header['iow'].format("?")
-            ret.append(self.curse_add_line(msg, optional=True, additional=True))
+            msg = self.layout_header[rorw].format("?")
+            ret = self.curse_add_line(msg, optional=True, additional=True)
+        return ret
+
+    def _get_process_curses_io_read(self, p, selected, args):
+        """Return process IO Read curses"""
+        return self._get_process_curses_io(p, selected, args, rorw='ior')
+
+    def _get_process_curses_io_write(self, p, selected, args):
+        """Return process IO Write curses"""
+        return self._get_process_curses_io(p, selected, args, rorw='iow')
+
+    def get_process_curses_data(self, p, selected, args):
+        """Get curses data to display for a process.
+
+        - p is the process to display
+        - selected is a tag=True if p is the selected process
+        """
+        ret = [self.curse_new_line()]
+        # When a process is selected:
+        # * display a special character at the beginning of the line
+        # * underline the command name
+        if args.is_standalone:
+            ret.append(self.curse_add_line(unicode_message('PROCESS_SELECTOR') if selected else ' ', 'SELECTED'))
+
+        # CPU
+        ret.append(self._get_process_curses_cpu(p, selected, args))
+
+        # MEM
+        ret.append(self._get_process_curses_mem(p, selected, args))
+        ret.append(self._get_process_curses_vms(p, selected, args))
+        ret.append(self._get_process_curses_rss(p, selected, args))
+
+        # PID
+        if not self.args.programs:
+            # Display processes, so the PID should be displayed
+            msg = self.layout_stat['pid'].format(p['pid'], width=self.__max_pid_size())
+        else:
+            # Display programs, so the PID should not be displayed
+            # Instead displays the number of children
+            msg = self.layout_stat['pid'].format(
+                len(p['childrens']) if 'childrens' in p else '_', width=self.__max_pid_size()
+            )
+        ret.append(self.curse_add_line(msg))
+
+        # USER
+        ret.append(self._get_process_curses_username(p, selected, args))
+
+        # TIME+
+        ret.append(self._get_process_curses_time(p, selected, args))
+
+        # THREAD
+        ret.append(self._get_process_curses_thread(p, selected, args))
+
+        # NICE
+        ret.append(self._get_process_curses_nice(p, selected, args))
+
+        # STATUS
+        ret.append(self._get_process_curses_status(p, selected, args))
+
+        # IO read/write
+        ret.append(self._get_process_curses_io_read(p, selected, args))
+        ret.append(self._get_process_curses_io_write(p, selected, args))
 
         # Command line
-        # If no command line for the process is available, fallback to
-        # the bare process name instead
-        if 'cmdline' in p:
-            cmdline = p['cmdline']
-        else:
-            cmdline = '?'
+        # If no command line for the process is available, fallback to the bare process name instead
+        bare_process_name = p['name']
+        cmdline = p.get('cmdline', '?')
+
         try:
+            process_decoration = 'PROCESS_SELECTED' if (selected and args.is_standalone) else 'PROCESS'
             if cmdline:
-                path, cmd, arguments = split_cmdline(cmdline)
+                path, cmd, arguments = split_cmdline(bare_process_name, cmdline)
+                # Manage end of line in arguments (see #1692)
+                arguments.replace('\r\n', ' ')
+                arguments.replace('\n', ' ')
                 if os.path.isdir(path) and not args.process_short_name:
                     msg = self.layout_stat['command'].format(path) + os.sep
                     ret.append(self.curse_add_line(msg, splittable=True))
-                    ret.append(self.curse_add_line(cmd, decoration='PROCESS', splittable=True))
+                    ret.append(self.curse_add_line(cmd, decoration=process_decoration, splittable=True))
                 else:
                     msg = self.layout_stat['command'].format(cmd)
-                    ret.append(self.curse_add_line(msg, decoration='PROCESS', splittable=True))
+                    ret.append(self.curse_add_line(msg, decoration=process_decoration, splittable=True))
                 if arguments:
                     msg = ' ' + self.layout_stat['command'].format(arguments)
                     ret.append(self.curse_add_line(msg, splittable=True))
             else:
-                msg = self.layout_stat['name'].format(p['name'])
-                ret.append(self.curse_add_line(msg, splittable=True))
+                msg = self.layout_stat['name'].format(bare_process_name)
+                ret.append(self.curse_add_line(msg, decoration=process_decoration, splittable=True))
         except (TypeError, UnicodeEncodeError) as e:
-            # Avoid crach after running fine for several hours #1335
+            # Avoid crash after running fine for several hours #1335
             logger.debug("Can not decode command line '{}' ({})".format(cmdline, e))
             ret.append(self.curse_add_line('', splittable=True))
 
         # Add extended stats but only for the top processes
-        if first and 'extended_stats' in p and args.enable_process_extended:
+        if args.cursor_position == 0 and 'extended_stats' in p and args.enable_process_extended:
             # Left padding
             xpad = ' ' * 13
             # First line is CPU affinity
@@ -345,8 +438,7 @@ class Plugin(GlancesPlugin):
                 msg = xpad + 'CPU affinity: ' + str(len(p['cpu_affinity'])) + ' cores'
                 ret.append(self.curse_add_line(msg, splittable=True))
             # Second line is memory info
-            if 'memory_info' in p and \
-               p['memory_info'] is not None:
+            if 'memory_info' in p and p['memory_info'] is not None:
                 ret.append(self.curse_new_line())
                 msg = '{}Memory info: {}'.format(xpad, p['memory_info'])
                 if 'memory_swap' in p and p['memory_swap'] is not None:
@@ -368,10 +460,8 @@ class Plugin(GlancesPlugin):
                 ret.append(self.curse_new_line())
                 msg = xpad + 'Open: ' + msg
                 ret.append(self.curse_add_line(msg, splittable=True))
-            # Fouth line is IO nice level (only Linux and Windows OS)
-            if 'ionice' in p and \
-               p['ionice'] is not None \
-               and hasattr(p['ionice'], 'ioclass'):
+            # Fourth line is IO nice level (only Linux and Windows OS)
+            if 'ionice' in p and p['ionice'] is not None and hasattr(p['ionice'], 'ioclass'):
                 ret.append(self.curse_new_line())
                 msg = xpad + 'IO nice: '
                 k = 'Class is '
@@ -423,11 +513,14 @@ class Plugin(GlancesPlugin):
 
         # Process list
         # Loop over processes (sorted by the sort key previously compute)
-        first = True
+        # This is a Glances bottleneck (see flame graph),
+        # get_process_curses_data should be optimzed
+        i = 0
         for p in self.__sort_stats(process_sort_key):
-            ret.extend(self.get_process_curses_data(p, first, args))
-            # End of extended stats
-            first = False
+            ret.extend(self.get_process_curses_data(p, i == args.cursor_position, args))
+            i += 1
+
+        # A filter is set Display the stats summaries
         if glances_processes.process_filter is not None:
             if args.reset_minmax_tag:
                 args.reset_minmax_tag = not args.reset_minmax_tag
@@ -456,12 +549,17 @@ class Plugin(GlancesPlugin):
         ret.append(self.curse_add_line(msg, optional=True))
         msg = self.layout_header['res'].format('RES')
         ret.append(self.curse_add_line(msg, optional=True))
-        msg = self.layout_header['pid'].format('PID', width=self.__max_pid_size())
+        if not self.args.programs:
+            msg = self.layout_header['pid'].format('PID', width=self.__max_pid_size())
+        else:
+            msg = self.layout_header['pid'].format('NPROCS', width=self.__max_pid_size())
         ret.append(self.curse_add_line(msg))
         msg = self.layout_header['user'].format('USER')
         ret.append(self.curse_add_line(msg, sort_style if process_sort_key == 'username' else 'DEFAULT'))
         msg = self.layout_header['time'].format('TIME+')
-        ret.append(self.curse_add_line(msg, sort_style if process_sort_key == 'cpu_times' else 'DEFAULT', optional=True))
+        ret.append(
+            self.curse_add_line(msg, sort_style if process_sort_key == 'cpu_times' else 'DEFAULT', optional=True)
+        )
         msg = self.layout_header['thread'].format('THR')
         ret.append(self.curse_add_line(msg))
         msg = self.layout_header['nice'].format('NI')
@@ -469,20 +567,31 @@ class Plugin(GlancesPlugin):
         msg = self.layout_header['status'].format('S')
         ret.append(self.curse_add_line(msg))
         msg = self.layout_header['ior'].format('R/s')
-        ret.append(self.curse_add_line(msg, sort_style if process_sort_key == 'io_counters' else 'DEFAULT', optional=True, additional=True))
+        ret.append(
+            self.curse_add_line(
+                msg, sort_style if process_sort_key == 'io_counters' else 'DEFAULT', optional=True, additional=True
+            )
+        )
         msg = self.layout_header['iow'].format('W/s')
-        ret.append(self.curse_add_line(msg, sort_style if process_sort_key == 'io_counters' else 'DEFAULT', optional=True, additional=True))
-        msg = self.layout_header['command'].format('Command')
+        ret.append(
+            self.curse_add_line(
+                msg, sort_style if process_sort_key == 'io_counters' else 'DEFAULT', optional=True, additional=True
+            )
+        )
+        if not self.args.programs:
+            msg = self.layout_header['command'].format('Command', "('k' to kill)" if args.is_standalone else "")
+        else:
+            msg = self.layout_header['command'].format('Programs', "('k' to kill)" if args.is_standalone else "")
         ret.append(self.curse_add_line(msg, sort_style if process_sort_key == 'name' else 'DEFAULT'))
 
     def __msg_curse_sum(self, ret, sep_char='_', mmm=None, args=None):
         """
         Build the sum message (only when filter is on) and add it to the ret dict.
 
-        * ret: list of string where the message is added
-        * sep_char: define the line separation char
-        * mmm: display min, max, mean or current (if mmm=None)
-        * args: Glances args
+        :param ret: list of string where the message is added
+        :param sep_char: define the line separation char
+        :param mmm: display min, max, mean or current (if mmm=None)
+        :param args: Glances args
         """
         ret.append(self.curse_new_line())
         if mmm is None:
@@ -490,24 +599,26 @@ class Plugin(GlancesPlugin):
             ret.append(self.curse_new_line())
         # CPU percent sum
         msg = self.layout_stat['cpu'].format(self.__sum_stats('cpu_percent', mmm=mmm))
-        ret.append(self.curse_add_line(msg,
-                                       decoration=self.__mmm_deco(mmm)))
+        ret.append(self.curse_add_line(msg, decoration=self.__mmm_deco(mmm)))
         # MEM percent sum
         msg = self.layout_stat['mem'].format(self.__sum_stats('memory_percent', mmm=mmm))
-        ret.append(self.curse_add_line(msg,
-                                       decoration=self.__mmm_deco(mmm)))
+        ret.append(self.curse_add_line(msg, decoration=self.__mmm_deco(mmm)))
         # VIRT and RES memory sum
-        if 'memory_info' in self.stats[0] and self.stats[0]['memory_info'] is not None and self.stats[0]['memory_info'] != '':
+        if (
+            'memory_info' in self.stats[0]
+            and self.stats[0]['memory_info'] is not None
+            and self.stats[0]['memory_info'] != ''
+        ):
             # VMS
-            msg = self.layout_stat['virt'].format(self.auto_unit(self.__sum_stats('memory_info', indice=1, mmm=mmm), low_precision=False))
-            ret.append(self.curse_add_line(msg,
-                                           decoration=self.__mmm_deco(mmm),
-                                           optional=True))
+            msg = self.layout_stat['virt'].format(
+                self.auto_unit(self.__sum_stats('memory_info', indice=1, mmm=mmm), low_precision=False)
+            )
+            ret.append(self.curse_add_line(msg, decoration=self.__mmm_deco(mmm), optional=True))
             # RSS
-            msg = self.layout_stat['res'].format(self.auto_unit(self.__sum_stats('memory_info', indice=0, mmm=mmm), low_precision=False))
-            ret.append(self.curse_add_line(msg,
-                                           decoration=self.__mmm_deco(mmm),
-                                           optional=True))
+            msg = self.layout_stat['res'].format(
+                self.auto_unit(self.__sum_stats('memory_info', indice=0, mmm=mmm), low_precision=False)
+            )
+            ret.append(self.curse_add_line(msg, decoration=self.__mmm_deco(mmm), optional=True))
         else:
             msg = self.layout_header['virt'].format('')
             ret.append(self.curse_add_line(msg))
@@ -534,23 +645,25 @@ class Plugin(GlancesPlugin):
         # IO read/write
         if 'io_counters' in self.stats[0] and mmm is None:
             # IO read
-            io_rs = int((self.__sum_stats('io_counters', 0) - self.__sum_stats('io_counters', indice=2, mmm=mmm)) / self.stats[0]['time_since_update'])
+            io_rs = int(
+                (self.__sum_stats('io_counters', 0) - self.__sum_stats('io_counters', indice=2, mmm=mmm))
+                / self.stats[0]['time_since_update']
+            )
             if io_rs == 0:
                 msg = self.layout_stat['ior'].format('0')
             else:
                 msg = self.layout_stat['ior'].format(self.auto_unit(io_rs, low_precision=True))
-            ret.append(self.curse_add_line(msg,
-                                           decoration=self.__mmm_deco(mmm),
-                                           optional=True, additional=True))
+            ret.append(self.curse_add_line(msg, decoration=self.__mmm_deco(mmm), optional=True, additional=True))
             # IO write
-            io_ws = int((self.__sum_stats('io_counters', 1) - self.__sum_stats('io_counters', indice=3, mmm=mmm)) / self.stats[0]['time_since_update'])
+            io_ws = int(
+                (self.__sum_stats('io_counters', 1) - self.__sum_stats('io_counters', indice=3, mmm=mmm))
+                / self.stats[0]['time_since_update']
+            )
             if io_ws == 0:
                 msg = self.layout_stat['iow'].format('0')
             else:
                 msg = self.layout_stat['iow'].format(self.auto_unit(io_ws, low_precision=True))
-            ret.append(self.curse_add_line(msg,
-                                           decoration=self.__mmm_deco(mmm),
-                                           optional=True, additional=True))
+            ret.append(self.curse_add_line(msg, decoration=self.__mmm_deco(mmm), optional=True, additional=True))
         else:
             msg = self.layout_header['ior'].format('')
             ret.append(self.curse_add_line(msg, optional=True, additional=True))
@@ -580,8 +693,8 @@ class Plugin(GlancesPlugin):
     def __sum_stats(self, key, indice=None, mmm=None):
         """Return the sum of the stats value for the given key.
 
-        * indice: If indice is set, get the p[key][indice]
-        * mmm: display min, max, mean or current (if mmm=None)
+        :param indice: If indice is set, get the p[key][indice]
+        :param mmm: display min, max, mean or current (if mmm=None)
         """
         # Compute stats summary
         ret = 0
@@ -628,10 +741,9 @@ class Plugin(GlancesPlugin):
             ret += str(indice)
         return ret
 
-    def __sort_stats(self, sortedby=None):
-        """Return the stats (dict) sorted by (sortedby)."""
-        return sort_stats(self.stats, sortedby,
-                          reverse=glances_processes.sort_reverse)
+    def __sort_stats(self, sorted_by=None):
+        """Return the stats (dict) sorted by (sorted_by)."""
+        return sort_stats(self.stats, sorted_by, reverse=glances_processes.sort_reverse)
 
     def __max_pid_size(self):
         """Return the maximum PID size in number of char."""
