@@ -2,20 +2,10 @@
 #
 # This file is part of Glances.
 #
-# Copyright (C) 2015 Nicolargo <nicolas@nicolargo.com>
+# SPDX-FileCopyrightText: 2022 Nicolas Hennion <nicolas@nicolargo.com>
 #
-# Glances is free software; you can redistribute it and/or modify
-# it under the terms of the GNU Lesser General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# SPDX-License-Identifier: LGPL-3.0-only
 #
-# Glances is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public License
-# along with this program. If not, see <http://www.gnu.org/licenses/>.
 
 """Glances Web Interface (Bottle based)."""
 
@@ -37,11 +27,11 @@ class GlancesWebServer(object):
             # Ignore kernel threads in process list
             glances_processes.disable_kernel_threads()
 
-        # Initial system informations update
+        # Initial system information update
         self.stats.update()
 
         # Init the Bottle Web server
-        self.web = GlancesBottle(args=args)
+        self.web = GlancesBottle(config=config, args=args)
 
     def serve_forever(self):
         """Main loop for the Web server."""
