@@ -3,20 +3,10 @@
 #
 # Glances - An eye on your system
 #
-# Copyright (C) 2019 Nicolargo <nicolas@nicolargo.com>
+# SPDX-FileCopyrightText: 2022 Nicolas Hennion <nicolas@nicolargo.com>
 #
-# Glances is free software; you can redistribute it and/or modify
-# it under the terms of the GNU Lesser General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# SPDX-License-Identifier: LGPL-3.0-only
 #
-# Glances is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public License
-# along with this program. If not, see <http://www.gnu.org/licenses/>.
 
 """Glances unitary tests suite."""
 
@@ -40,7 +30,7 @@ from glances.secure import secure_popen
 from glances.compat import PY3
 
 if PY3:
-    from tracemalloc import Snapshot
+    pass
 
 # Global variables
 # =================
@@ -287,7 +277,7 @@ class TestGlances(unittest.TestCase):
     def test_017_programs(self):
         """Check Programs function (it's not a plugin)."""
         # stats_to_check = [ ]
-        print('INFO: [TEST_010] Check PROGRAM stats')
+        print('INFO: [TEST_017] Check PROGRAM stats')
         stats_grab = processes_to_programs(stats.get_plugin('processlist').get_raw())
         self.assertTrue(type(stats_grab) is list, msg='Programs stats is not a list')
         print('INFO: PROGRAM list stats: %s items in the list' % len(stats_grab))
@@ -332,7 +322,7 @@ class TestGlances(unittest.TestCase):
                             msg='{} view is not a dict'.format(plugin))
 
     def test_097_attribute(self):
-        """Test GlancesAttribute classe"""
+        """Test GlancesAttribute classes"""
         print('INFO: [TEST_097] Test attribute')
         # GlancesAttribute
         from glances.attribute import GlancesAttribute
@@ -355,7 +345,7 @@ class TestGlances(unittest.TestCase):
         self.assertEqual(a.history_mean(nb=3), 4.5)
 
     def test_098_history(self):
-        """Test GlancesHistory classe"""
+        """Test GlancesHistory classes"""
         print('INFO: [TEST_098] Test history')
         # GlancesHistory
         from glances.history import GlancesHistory
